/******************************************************************************
globus_poll.c

Description:

  A general polling infrastructure

CVS Information:

  $Source: /home/globdev/CVS/globus-1998_04_16/Globus/Miscellaneous/common/library/globus_poll.c,v $
  $Date: 1998/10/07 02:04:34 $
  $Revision: 1.6 $
  $State: Exp $
  $Author: bester $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "globus_common.h"


/******************************************************************************
		       Define module specific variables
******************************************************************************/
globus_list_t *		globus_poll_list;
globus_mutex_t		globus_poll_mutex;

#ifndef BUILD_LITE
globus_cond_t		globus_l_poll_cond;
static globus_bool_t	globus_l_poll_using_handler_thread = GLOBUS_FALSE;
static globus_bool_t	globus_l_poll_handler_thread_finish = GLOBUS_FALSE;
static globus_bool_t	globus_l_poll_handler_thread_not_terminated = GLOBUS_TRUE;
static globus_thread_t	globus_l_poll_handler_thread;
static globus_mutex_t	globus_l_poll_handler_mutex;
static void 	       *globus_l_poll_handler_thread_func(void *);
#endif

/******************************************************************************
			       Module definition
******************************************************************************/
static int
globus_l_poll_activate(void);

static int
globus_l_poll_deactivate(void);

globus_module_descriptor_t		globus_i_poll_module =
{
    "globus_poll",
    globus_l_poll_activate,
    globus_l_poll_deactivate,
    GLOBUS_NULL
};

/******************************************************************************
		       globus_poll function definitions
******************************************************************************/

/*
 * globus_poll_activate()
 */
static int
globus_l_poll_activate(void)
{
    globus_mutex_init(&globus_poll_mutex, (globus_mutexattr_t *) GLOBUS_NULL);
    globus_poll_list = GLOBUS_NULL;

#ifndef BUILD_LITE
    globus_module_activate(GLOBUS_THREAD_MODULE);
    globus_cond_init(&globus_l_poll_cond,
		     (globus_condattr_t *) GLOBUS_NULL);
    globus_mutex_init(&globus_l_poll_handler_mutex,
		      (globus_mutexattr_t *) GLOBUS_NULL);
#endif
    
    return GLOBUS_SUCCESS;
}
/* globus_poll_activate() */

/*
 * globus_poll_deactivate()
 */
static int
globus_l_poll_deactivate(void)
{
#ifndef BUILD_LITE
    globus_mutex_lock(&globus_poll_mutex);
    globus_l_poll_handler_thread_finish = GLOBUS_TRUE;
    globus_mutex_unlock(&globus_poll_mutex);

    globus_mutex_lock(&globus_poll_mutex);
    while(globus_l_poll_using_handler_thread &&
	  globus_l_poll_handler_thread_not_terminated)
    {
	globus_cond_wait(&globus_l_poll_cond,
			 &globus_poll_mutex);
    }
    globus_mutex_unlock(&globus_poll_mutex);
#endif

    globus_mutex_destroy(&globus_poll_mutex);
    globus_list_free(globus_poll_list);

#ifndef BUILD_LITE
    globus_module_deactivate(GLOBUS_THREAD_MODULE);
#endif

    return GLOBUS_SUCCESS;
}
/* globus_poll_deactivate() */


/*
 * globus_l_poll_nonblocking()
 */
globus_bool_t
globus_l_poll_nonblocking(void)
{
    globus_list_t *			list;
    globus_bool_t			rc;

    rc = GLOBUS_FALSE;

    list = globus_poll_list;
    while(!globus_list_empty(list))
    {
        globus_poll_func_t		polling_func;

        polling_func = (globus_poll_func_t) globus_list_first(list);
        list = globus_list_rest(list);

        if (polling_func(GLOBUS_FALSE))
        {
            rc = GLOBUS_TRUE;
        }
	globus_thread_yield();
    }
    return rc;
}

/*
 * globus_poll_blocking()
 */
void
globus_poll_blocking(void)
{
#   if !defined(BUILD_LITE)
    {
        globus_thread_yield();
        return;    /* polling happens in separate thread */
    }
#   else
    {
        globus_mutex_lock(&globus_poll_mutex);
        {
	    if (!globus_list_empty(globus_poll_list)
	        && globus_list_empty(globus_list_rest(globus_poll_list)))
	    {
	        globus_poll_func_t		polling_func;

	        polling_func = (globus_poll_func_t)
		    globus_list_first(globus_poll_list);

	        /*
	         * The mutex should probably be release here, but then we need to
	         * deal with the issue of what happens if the user adds another
	         * polling function or removes the current one.  Do we have to
	         * somehow wake up the current blocking poll so that the
	         * adjustments can be made?
	         *
	         * For now, to keep things simple, I have defined the routines such
	         * that the poll list cannot be modified when a polling operation
	         * is in progress.  Perhaps this should be a topic of discussion at
	         * some point.  --BRT
	         */
	        while(polling_func(GLOBUS_TRUE) == GLOBUS_FALSE);
	    }
	    else
	    {
	        while(!globus_list_empty(globus_poll_list) &&
		      globus_l_poll_nonblocking() == GLOBUS_FALSE);
	    }
        }
        globus_mutex_unlock(&globus_poll_mutex);
    }
#   endif
}
/* globus_poll_blocking() */

/*
 * globus_poll_nonblocking()
 */
void
globus_poll_nonblocking(void)
{
#   if defined(BUILD_LITE)
    {
        globus_mutex_lock(&globus_poll_mutex);
        {
	    globus_l_poll_nonblocking();
        }
        globus_mutex_unlock(&globus_poll_mutex);
    }
#   else
    {
        globus_thread_yield();
    }
#   endif
}
/* globus_poll_nonblocking() */

/*
 * globus_poll_add()
 */
int
globus_poll_add(
    globus_poll_func_t			polling_func,
    int					frequency)
{
    int					rc;

    globus_mutex_lock(&globus_poll_mutex);
    {
	globus_list_t *			entry;

	rc = GLOBUS_SUCCESS;
	
	entry = globus_list_search(globus_poll_list, (void *) polling_func);
	if (entry == GLOBUS_NULL)
	{
	    if (globus_list_insert(&globus_poll_list,
				   (void *) polling_func) != GLOBUS_SUCCESS)
	    {
		rc = GLOBUS_FAILURE;
	    }
	}
#ifndef BUILD_LITE
	if(rc == GLOBUS_SUCCESS &&
	   globus_l_poll_using_handler_thread == GLOBUS_FALSE &&
	   globus_l_poll_handler_thread_finish == GLOBUS_FALSE)
	{
	    globus_thread_create(&globus_l_poll_handler_thread,
				 (globus_threadattr_t *) GLOBUS_NULL,
				 globus_l_poll_handler_thread_func,
				 GLOBUS_NULL);
	    globus_l_poll_using_handler_thread = GLOBUS_TRUE;
	}
#endif
    }

    globus_mutex_unlock(&globus_poll_mutex);

    return rc;
}
/* globus_poll_add() */

/*
 * globus_poll_remove()
 */
int
globus_poll_remove(
    globus_poll_func_t			polling_func)
{
    int					rc;
    
    globus_mutex_lock(&globus_poll_mutex);
    {
	globus_list_t *			entry;
    
	entry = globus_list_search(globus_poll_list, (void *) polling_func);
	if (entry != GLOBUS_NULL)
	{
	    globus_list_remove(&globus_poll_list, entry);
	
	    rc = GLOBUS_SUCCESS;
	}
	else
	{
	    rc = GLOBUS_FAILURE;
	}
    }
    globus_mutex_unlock(&globus_poll_mutex);

    return rc;
}
/* globus_poll_remove() */

#ifndef BUILD_LITE
static void *
globus_l_poll_handler_thread_func(void *arg)
{
    globus_mutex_lock(&globus_poll_mutex);

    while(globus_list_empty(globus_poll_list) == GLOBUS_FALSE &&
	  globus_l_poll_handler_thread_finish == GLOBUS_FALSE)
    {
        /* this needs to be called with the mutex locked */
	globus_l_poll_nonblocking();

        /* this is needed to allow users to register their own
	   poll functions
	 */
	globus_mutex_unlock(&globus_poll_mutex);
	globus_thread_yield();
	globus_mutex_lock(&globus_poll_mutex);
    }

    globus_l_poll_using_handler_thread = GLOBUS_FALSE;
    globus_l_poll_handler_thread_not_terminated = GLOBUS_FALSE;
    globus_mutex_unlock(&globus_poll_mutex);
    globus_cond_signal(&globus_l_poll_cond);

    return GLOBUS_NULL;
} /* globus_l_poll_handler_thread_func() */
#endif
