/*
 *
 * External header for a C-threads based Ports0 threads library.
 * This will be included by user code.
 *
 */

#if !defined(GLOBUS_INCLUDE_GLOBUS_THREAD)
#define GLOBUS_INCLUDE_GLOBUS_THREAD 1

#include <mach/cthreads.h>

typedef struct cthread		globus_thread_t;
typedef int			globus_threadattr_t;
typedef int			globus_mutexattr_t;
typedef int			globus_condattr_t;
typedef struct mutex		globus_mutex_t
typedef struct condition 	globus_cond_t

#define GLOBUS_THREAD_ONCE_INIT 0
typedef int globus_thread_once_t;
extern  int globus_i_thread_actual_thread_once(
    globus_thread_once_t *once_control,
    void (*init_routine)(void));

typedef int globus_thread_key_t;
typedef void (*globus_thread_key_destructor_func_t)(void *value);

#ifndef ENOSYS
/* On the NeXT this is only defined with _POSIX_SOURCE */
#define ENOSYS 87
#endif

/*
 * Some externs...
 */
extern int		globus_thread_create(globus_thread_t *thread,
					     globus_threadattr_t *attr,
					     globus_thread_func_t func,
					     void *user_arg);
extern void		globus_thread_exit(void *status);

extern int		globus_thread_key_create(globus_thread_key_t *key,
				globus_thread_key_destructor_func_t func);
extern int		globus_thread_setspecific(globus_thread_key_t key,
						  void *value);
extern void *           globus_thread_getspecific(globus_thread_key_t key);

#define globus_threadattr_init(attr)	0 /* successful return */
#define globus_threadattr_destroy(attr)	0 /* successful return */
#define globus_threadattr_setstacksize(attr, stacksize) \
	ENOSYS /* unsuccessful return - not implemented */
#define globus_threadattr_getstacksize(attr, stacksize) \
	ENOSYS /* unsuccessful return - not implemented */
#define globus_thread_key_delete(key)	0 /* successful return */


/*
 * macros and prototypes for thread manipulation routines
 */
#define globus_macro_thread_yield() \
    cthread_yield()
#define globus_macro_thread_self() \
    (*(globus_thread_t *)(cthread_self()))
#define globus_macro_thread_equal(t1,t2) \
    ((int)&(t1) == (int)&(t2))
#define globus_macro_thread_once(once_control, init_routine) \
    (*once_control ? 0 : globus_i_thread_actual_thread_once(once_control, init_routine))
#ifndef USE_MACROS
extern void		globus_thread_yield(void);
extern globus_thread_t	globus_thread_self(void);
extern int		globus_thread_equal(globus_thread_t t1,
					   globus_thread_t t2);
extern int		globus_thread_once(globus_thread_once_t *once_control,
					  void (*init_routine)(void));
extern globus_bool_t globus_thread_i_am_only_thread(void);
#else  /* USE_MACROS */
#define globus_thread_yield() \
    globus_macro_thread_yield()
#define globus_thread_self() \
    globus_macro_thread_self()
#define globus_thread_equal(t1,t2) \
    globus_macro_thread_equal(t1,t2)
#define globus_thread_once(once_control, init_routine) \
    globus_macro_thread_once(once_control, init_routine)
#define globus_thread_i_am_only_thread() GLOBUS_FALSE
#endif /* USE_MACROS */


/*
 * lock macros and prototypes
 */

#define globus_macro_mutex_init(mut, attr) \
    (mutex_init(mut), 0)
#ifdef TARGET_ARCH_NEXTSTEP
#define globus_macro_mutex_destroy(mutex)	0 /* successful return */
#define globus_macro_mutex_lock(mut) \
    (mutex_try_lock(mut) ? 0 : \
     (mutex_wait_lock(mut), 0) )
#else  /* TARGET_ARCH_NEXTSTEP */
#define globus_macro_mutex_destroy(mut)	mutex_clear(mut)
#define globus_macro_mutex_lock(mut) \
    (mutex_lock(mut), 0)
#endif /* TARGET_ARCH_NEXTSTEP */
#define globus_macro_mutex_trylock(mut) \
    (mutex_try_lock(mut) ? 0 : EBUSY)
#define globus_macro_mutex_unlock(mut) \
    (mutex_unlock(mut), 0)
#define globus_mutexattr_init(attr) 0 /* successful return */
#define globus_mutexattr_destroy(attr) 0 /* successful return */

#ifndef USE_MACROS
extern int		globus_mutex_init(globus_mutex_t *mutex,
					  globus_mutexattr_t *attr);
extern int		globus_mutex_destroy(globus_mutex_t *mutex);
extern int		globus_mutex_lock(globus_mutex_t *mutex);
extern int		globus_mutex_trylock(globus_mutex_t *mutex);
extern int		globus_mutex_unlock(globus_mutex_t *mutex);
#else  /* USE_MACROS */
#define globus_mutex_init(mutex, attr) \
    globus_macro_mutex_init(mutex, attr)
#define globus_mutex_destroy(mutex) \
    globus_macro_mutex_destroy(mutex)
#define globus_mutex_lock(mutex) \
    globus_macro_mutex_lock(mutex)
#define globus_mutex_trylock(mutex) \
    globus_macro_mutex_trylock(mutex)
#define globus_mutex_unlock(mutex) \
    globus_macro_mutex_unlock(mutex)
#endif /* USE_MACROS */


/*
 * condition macros and prototypes
 */
#define globus_macro_cond_wait(cv, mut) \
    (condition_wait(cv, mut), 0)
#define globus_condattr_init(attr) 0 /* successful return */
#define globus_condattr_destroy(attr) 0 /* successful return */

#ifndef USE_MACROS
extern int	globus_cond_wait(globus_cond_t *cond,
				 globus_mutex_t *mutex);
#else /* USE_MACROS */
#define globus_cond_wait(cv, mut) \
    globus_macro_cond_wait(cv, mut)
#endif /* USE_MACROS */

#ifdef TARGET_ARCH_NEXTSTEP
/* These cannot be done within a comma operator to get the right return code */
#define globus_macro_cond_init(cv, attr) \
    condition_init(cv)
#define globus_macro_cond_destroy(cv) \
    condition_clear(cv)
#define globus_macro_cond_signal(cv) \
    condition_signal(cv)
#define globus_macro_cond_broadcast(cv) \
    condition_broadcast(cv)
#else  /* !TARGET_ARCH_NEXTSTEP */
#define globus_macro_cond_init(cv, attr) \
    (condition_init(cv), 0)
#define globus_macro_cond_destroy(cv) \
    (condition_clear(cv), 0)
#define globus_macro_cond_signal(cv) \
    (condition_signal(cv), 0)
#define globus_macro_cond_broadcast(cv) \
    (condition_broadcast(cv), 0)
#endif /* !TARGET_ARCH_NEXTSTEP */

#if !defined(USE_MACROS) || defined(TARGET_ARCH_NEXTSTEP)
extern int	globus_cond_init(globus_cond_t *cond, globus_condattr_t *attr);
extern int	globus_cond_destroy(globus_cond_t *cond);
extern int	globus_cond_signal(globus_cond_t *cond);
extern int	globus_cond_broadcast(globus_cond_t *cond);
#else
#define globus_cond_init(cv) \
    globus_macro_cond_init(cv)
#define globus_cond_destroy(cv) \
    globus_macro_cond_destroy(cv)
#define globus_cond_signal(cv) \
    globus_macro_cond_signal(cv)
#define globus_cond_broadcast(cv) \
    globus_macro_cond_broadcast(cv)
#endif

#endif /* GLOBUS_INCLUDE_GLOBUS_THREAD */
